#ifndef _BusyArray_h_
#define _BusyArray_h_

// BusyArray
//
// Array that keeps an ordered list (by time) of the occupation
// of a node, with a BusyDate for each time, representing how many
// threads (and which states) were busy between that time and the
// next time of the list. Receives PStates as input objects.
// Its enumerator returns PStates created to represent each of these
// time slices.

// 19980227 BS  creation

#include <Foundation/Foundation.h>
#include "../General.bproj/PSortedArray.h"
#include "BusyDate.h"
#include "../General.bproj/Protocols.h"
#include "../General.bproj/PajeContainer.h"

@interface BusyArray : PSortedArray
{
    PajeContainer *container;
    PajeEntityType *entityType;
}

- (id)initWithEntityType:(PajeEntityType *)et
               container:(PajeContainer *)cont
               startTime:(NSDate *)startTime
                 endTime:(NSDate *)endTime;
- (id)initWithEntityType:(PajeEntityType *)et
               container:(PajeContainer *)cont
               startTime:(NSDate *)startTime
                 endTime:(NSDate *)endTime
              enumerator:(NSEnumerator *)enumerator
              nameFilter:(NSSet *)filter;

- (void)dealloc;

- (void)addObject:(id <PajeEntity>)obj;

- (NSEnumerator *)objectEnumeratorOfClass:(Class)c;
- (NSEnumerator *)objectEnumeratorOfClass:(Class)c
                                 fromTime:(NSDate *)t1 toTime:(NSDate *)t2;

- (PajeContainer *)container;
- (PajeEntityType *)entityType;
- (NSDate *)startTime;
- (NSDate *)endTime;
@end

#endif
