#ifndef _ReduceEntity_h_
#define _ReduceEntity_h_

//
// An entity that represents many others, reducing them.
// Subclasses can reduce by counting, summing, maxing, etc.
// Reduced entities are kept in an array that has also some global data.
// This entity only has a pointer to the array and an index to the correct entry.
//
// BS 2001.02.20 creation
//

#include "BusyArray.h"
#include "../General.bproj/PajeEntity.h"
#include "../General.bproj/PajeFilter.h"

@interface ReduceEntity : PajeEntity
{
    BusyArray *array;
    int index;
}

+ (ReduceEntity *)entityWithArray:(BusyArray *)a index:(int)i;

- (id)initWithArray:(BusyArray *)a index:(int)i;

- (PajeDrawingType)drawingType;
- (PajeEntityType *)entityType;

- (NSDate *)startTime;
- (NSDate *)endTime;
- (PajeContainer *)container;
- (NSString *)name;
- (NSColor *)color;
- (NSArray *)relatedEntities;

+ (void)getMinValue:(NSNumber **)min
           maxValue:(NSNumber **)max
           forArray:(BusyArray *)a
      pajeComponent:(PajeFilter *)filter;

- (NSNumber *)value;

// to be implemented by subclasses
+ (NSString *)titleForPopUp;
+ (NSNumber *)valueForRelatedEntities:(NSArray *)entities
                        pajeComponent:(PajeFilter *)filter;
@end

@interface CountReduceEntity : ReduceEntity
+ (NSString *)titleForPopUp;
+ (NSNumber *)valueForRelatedEntities:(NSArray *)entities
                        pajeComponent:(PajeFilter *)filter;
@end
@interface SumReduceEntity : ReduceEntity
+ (NSString *)titleForPopUp;
+ (NSNumber *)valueForRelatedEntities:(NSArray *)entities
                        pajeComponent:(PajeFilter *)filter;
@end
@interface AverageReduceEntity : ReduceEntity
+ (NSString *)titleForPopUp;
+ (NSNumber *)valueForRelatedEntities:(NSArray *)entities
                        pajeComponent:(PajeFilter *)filter;
@end

#endif
