#ifndef _PajeFilter_h_
#define _PajeFilter_h_

/*
 * PajeFilter.h
 *
 * Base class of Paje filters.
 * Keeps the graph of filters;
 * forwards all notifications to following component;
 * forwards all queries to preceeding component.
 *
 * created by benhur on 11 may 1999
 */

#include <AppKit/AppKit.h>
#include "Protocols.h"
#include "MiscTree+Additions.h"
#include "PajeContainer.h"
//#include "../Paje/PajeTraceController.h"
@class PajeTraceController;

//
// Notifications
//

@class PajeFilter;
@interface PajeComponent : NSObject
{
    PajeFilter *outputComponent;
    PajeFilter *inputComponent;
    SEL outputSelector;
    PajeTraceController *controller;
}

+ (PajeComponent *)componentWithController:(PajeTraceController *)c;
- (id)initWithController:(PajeTraceController *)c;

- (void)setInputComponent:(PajeComponent *)component;
- (void)setOutputComponent:(PajeComponent *)component;

- (void)inputEntity:(id)entity;
- (void)outputEntity:(id)entity;

- (NSString *)traceDescription; /* only for use by simulator */

- (void)registerFilter:(PajeFilter *)filter;
- (void)registerTool:(PajeFilter *)filter;

- (NSString *)filterName;
- (NSView *)filterView;
- (NSString *)filterKeyEquivalent;
- (id)filterDelegate;
@end


@interface PajeFilter : PajeComponent
{
}

//
// Deal with notifications
//
- (void)dataChangedForEntityType:(PajeEntityType *)entityType;
- (void)colorChangedForEntityType:(PajeEntityType *)entityType;
- (void)hierarchyChanged;

- (void)filterContainer:(PajeContainer *)container show:(BOOL)show;

//
// Deal with queries
//

- (NSDate *)startTime;
- (NSDate *)endTime;
//
// Accessing entities
//
- (PajeContainer *)rootInstance;

- (MiscTree *)hierarchyTree;

- (NSEnumerator *)enumeratorOfEntitiesTyped:(PajeEntityType *)entityType
                                inContainer:(PajeContainer *)container
                                   fromTime:(NSDate *)start
                                     toTime:(NSDate *)end;

- (NSEnumerator *)enumeratorOfContainersTyped:(PajeContainerType *)entityType
                                  inContainer:(PajeContainer *)container;

- (NSArray *)allNamesForEntityType:(PajeEntityType *)entityType;

- (NSNumber *)minValueForEntityType:(PajeEntityType *)entityType;
- (NSNumber *)maxValueForEntityType:(PajeEntityType *)entityType;
- (NSNumber *)minValueForEntityType:(PajeEntityType *)entityType
                        inContainer:(PajeContainer *)container;
- (NSNumber *)maxValueForEntityType:(PajeEntityType *)entityType
                        inContainer:(PajeContainer *)container;

- (PajeDrawingType)drawingTypeForEntityType:(PajeEntityType *)entityType;
- (NSArray *)fieldNamesForEntityType:(PajeEntityType *)entityType;
- (NSArray *)fieldNamesForEntityType:(PajeEntityType *)entityType
                                name:(NSString *)name;

// Colors
// - when each entity name has a different color
- (NSColor *)colorForName:(NSString *)name
             ofEntityType:(PajeEntityType *)entityType;
- (void)setColor:(NSColor *)color
         forName:(id)name
    ofEntityType:(PajeEntityType *)entityType;
// - when there's one color for entity type (variables)
- (NSColor *)colorForEntityType:(PajeEntityType *)entityType;
- (void)setColor:(NSColor *)color
   forEntityType:(PajeEntityType *)entityType;
- (void)setColor:(NSColor *)color forEntity:(id<PajeEntity>)entity;

//
// Getting info from entity
//
- (NSArray *)fieldNamesForEntity:(id<PajeEntity>)entity;
- (id)valueOfFieldNamed:(NSString *)fieldName forEntity:(id<PajeEntity>)entity;

- (PajeContainer *)containerForEntity:(id<PajeEntity>)entity;
- (PajeEntityType *)entityTypeForEntity:(id<PajeEntity>)entity;
- (PajeContainer *)sourceContainerForEntity:(id<PajeLink>)entity;
- (PajeEntityType *)sourceEntityTypeForEntity:(id<PajeLink>)entity;
- (PajeContainer *)destContainerForEntity:(id<PajeLink>)entity;
- (PajeEntityType *)destEntityTypeForEntity:(id<PajeLink>)entity;
- (NSArray *)relatedEntitiesForEntity:(id<PajeEntity>)entity;
- (NSColor *)colorForEntity:(id<PajeEntity>)entity;
- (NSDate *)startTimeForEntity:(id<PajeEntity>)entity;
- (NSDate *)endTimeForEntity:(id<PajeEntity>)entity;
- (NSDate *)timeForEntity:(id<PajeEntity>)entity;
- (PajeDrawingType)drawingTypeForEntity:(id<PajeEntity>)entity;
- (NSString *)nameForEntity:(id<PajeEntity>)entity;
- (NSNumber *)valueForEntity:(id<PajeEntity>)entity; // for variables
- (NSString *)descriptionForEntity:(id<PajeEntity>)entity;
- (int)imbricationLevelForEntity:(id<PajeEntity>)entity;

- (BOOL)canHighlightEntity:(id<PajeEntity>)entity;


- (void)verifyStartTime:(NSDate *)start endTime:(NSDate *)end;


//
// Inspecting an entity
//
- (void)inspectEntity:(id<PajeInspecting>)entity;

@end

#endif
