#ifndef _STEntityDescriptor_h_
#define _STEntityDescriptor_h_

// Paje
// ----
// STEntityDescriptor.h
// holds a description of a type of entity, with information
// on how to draw one entity of this type.

// 18.apr.1998 BS  creation


#include <Foundation/Foundation.h>
#include "../General.bproj/Protocols.h"
#include "../General.bproj/PajeContainer.h"

typedef     void (pathfunction)(float x, float y, float w, float h);
typedef     void (drawfunction)(void);

@interface STEntityDescriptor : NSObject
{
    id entityType;   // thread state, thread event, semaphore state, etc
    PajeDrawingType drawingType;  // Paje{Event,State,Link}DrawingType
    NSRect drawingRect;
    // rect with the position and size to be added to the position and size
    // calculated to the entity (e.g., events are normally bigger than a point).
    pathfunction *pathFunction;
    // pointer to function that creates a path of the entity representation.
    drawfunction *drawFunction;
    // pointer to function that draws a path created by pathFunction.
    drawfunction *highlightFunction;
    // pointer to function that highlights a path created by pathFunction.
    NSMutableDictionary *containerToRect;
    NSMutableDictionary *instanceToRect;
    BOOL canDisplay;
    float insetAmount;
}

+ (drawfunction *)drawFunctionForName:(NSString*)name;
+ (pathfunction *)shapeFunctionForName:(NSString*)name;
+ (NSString *)nameForDrawFunction:(drawfunction *)f;
+ (NSString *)nameForShapeFunction:(pathfunction *)f;
+ (NSEnumerator *)shapeFunctionsValueEnumerator;
+ (NSEnumerator *)drawFunctionsValueEnumerator;

+ (STEntityDescriptor *)descriptorWithEntityType:(PajeEntityType *)etype
                                     drawingType:(PajeDrawingType)dtype;
- (id)initWithEntityType:(PajeEntityType *)etype
             drawingType:(PajeDrawingType)dtype;

- (void)dealloc;

// forget everything
- (void)reset;

// accessor methods
- (PajeEntityType *)entityType;
- (PajeDrawingType)drawingType;

- (void)setDrawingRect:(NSRect)r;
- (NSRect)drawingRect;

- (void)setPathFunction:(pathfunction *)f;
- (void)setDrawFunction:(drawfunction *)f;
- (void)setHighlightFunction:(drawfunction *)f;
- (pathfunction *)pathFunction;
- (drawfunction *)drawFunction;
- (drawfunction *)highlightFunction;

// Number of points one entity should be less than another when imbricated.
- (float)insetAmount;

- (BOOL)canDisplay;
- (void)setCanDisplay:(BOOL)flag;

- (void)setRect:(NSRect)rect forContainer:(PajeContainer *)container;
- (NSRect)rectInContainer:(PajeContainer *)container;
- (BOOL)isContainer:(PajeContainer *)container inRect:(NSRect)rect;
- (BOOL)isPoint:(NSPoint)point inContainer:(PajeContainer *)container;

- (void)setRect:(NSRect)rect forInstance:(id)entity;
- (NSRect)rectForInstance:(id)entity;
- (BOOL)isInstance:(id)entity inRect:(NSRect)rect;
- (BOOL)isPoint:(NSPoint)point inInstance:(id)entity;

- (NSEnumerator *)instanceEnumerator;

- (float)offset;
- (void)setOffset:(float)value;
- (float)separation;
- (void)setSeparation:(float)value;

// from NSObject protocol
- (unsigned int)hash;
- (BOOL)isEqual:(id)other;
- (NSString *)description;
@end

#endif
